#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<GelfWriter>::TypeImpl()
{ }

TypeImpl<GelfWriter>::~TypeImpl()
{ }

String TypeImpl<GelfWriter>::GetName() const
{
	return "GelfWriter";
}

int TypeImpl<GelfWriter>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<GelfWriter>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<GelfWriter>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 99:
			if (name == "connected")
				return offset + 4;

			break;
		case 101:
			if (name == "enable_send_perfdata")
				return offset + 3;

			break;
		case 104:
			if (name == "host")
				return offset + 0;

			break;
		case 112:
			if (name == "port")
				return offset + 1;

			break;
		case 115:
			if (name == "source")
				return offset + 2;
			if (name == "should_connect")
				return offset + 5;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<GelfWriter>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 2, 0};
		case 1:
			return {1, "String", "port", "port", nullptr, 2, 0};
		case 2:
			return {2, "String", "source", "source", nullptr, 2, 0};
		case 3:
			return {3, "Number", "enable_send_perfdata", "enable_send_perfdata", nullptr, 2, 0};
		case 4:
			return {4, "Number", "connected", "connected", nullptr, 1025, 0};
		case 5:
			return {5, "Number", "should_connect", "should_connect", nullptr, 1025, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<GelfWriter>::GetFieldCount() const
{
	return 6 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<GelfWriter>::GetFactory() const
{
	return TypeHelper<GelfWriter, false>::GetFactory();
}

std::vector<String> TypeImpl<GelfWriter>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<GelfWriter>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<GelfWriter>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<GelfWriter>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<GelfWriter>::OnPortChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<GelfWriter>::OnSourceChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<GelfWriter>::OnEnableSendPerfdataChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<GelfWriter>::OnConnectedChanged.connect(callback);
			break;
		case 5:
			ObjectImpl<GelfWriter>::OnShouldConnectChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<GelfWriter>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<String>([this]() { return GetPort(); }), utils);
	if (2 & types)
		ValidateSource(Lazy<String>([this]() { return GetSource(); }), utils);
	if (2 & types)
		ValidateEnableSendPerfdata(Lazy<bool>([this]() { return GetEnableSendPerfdata(); }), utils);
	if (1 & types)
		ValidateConnected(Lazy<bool>([this]() { return GetConnected(); }), utils);
	if (1 & types)
		ValidateShouldConnect(Lazy<bool>([this]() { return GetShouldConnect(); }), utils);
}

void ObjectImpl<GelfWriter>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<GelfWriter>::SimpleValidatePort(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<GelfWriter>::SimpleValidateSource(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<GelfWriter>::SimpleValidateEnableSendPerfdata(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<GelfWriter>::SimpleValidateConnected(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<GelfWriter>::SimpleValidateShouldConnect(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<GelfWriter>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPort(GetDefaultPort(), true);
	SetSource(GetDefaultSource(), true);
	SetEnableSendPerfdata(GetDefaultEnableSendPerfdata(), true);
	SetConnected(GetDefaultConnected(), true);
	SetShouldConnect(GetDefaultShouldConnect(), true);
}

ObjectImpl<GelfWriter>::~ObjectImpl()
{ }

void ObjectImpl<GelfWriter>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPort(value, suppress_events, cookie);
			break;
		case 2:
			SetSource(value, suppress_events, cookie);
			break;
		case 3:
			SetEnableSendPerfdata(value, suppress_events, cookie);
			break;
		case 4:
			SetConnected(value, suppress_events, cookie);
			break;
		case 5:
			SetShouldConnect(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<GelfWriter>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPort();
		case 2:
			return GetSource();
		case 3:
			return GetEnableSendPerfdata();
		case 4:
			return GetConnected();
		case 5:
			return GetShouldConnect();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<GelfWriter>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePort(lvalue, utils);
			break;
		case 2:
			ValidateSource(lvalue, utils);
			break;
		case 3:
			ValidateEnableSendPerfdata(lvalue, utils);
			break;
		case 4:
			ValidateConnected(lvalue, utils);
			break;
		case 5:
			ValidateShouldConnect(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<GelfWriter>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPort(cookie);
			break;
		case 2:
			NotifySource(cookie);
			break;
		case 3:
			NotifyEnableSendPerfdata(cookie);
			break;
		case 4:
			NotifyConnected(cookie);
			break;
		case 5:
			NotifyShouldConnect(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<GelfWriter>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<GelfWriter>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<GelfWriter>::GetPort() const
{
	return m_Port;
}

String ObjectImpl<GelfWriter>::GetSource() const
{
	return m_Source;
}

bool ObjectImpl<GelfWriter>::GetEnableSendPerfdata() const
{
	return m_EnableSendPerfdata;
}

bool ObjectImpl<GelfWriter>::GetConnected() const
{
	return m_Connected;
}

bool ObjectImpl<GelfWriter>::GetShouldConnect() const
{
	return m_ShouldConnect;
}

void ObjectImpl<GelfWriter>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<GelfWriter>::SetPort(const String& value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<GelfWriter>::SetSource(const String& value, bool suppress_events, const Value& cookie)
{
	m_Source = value;
	if (!suppress_events)
		NotifySource(cookie);
}

void ObjectImpl<GelfWriter>::SetEnableSendPerfdata(bool value, bool suppress_events, const Value& cookie)
{
	m_EnableSendPerfdata = value;
	if (!suppress_events)
		NotifyEnableSendPerfdata(cookie);
}

void ObjectImpl<GelfWriter>::SetConnected(bool value, bool suppress_events, const Value& cookie)
{
	m_Connected = value;
	if (!suppress_events)
		NotifyConnected(cookie);
}

void ObjectImpl<GelfWriter>::SetShouldConnect(bool value, bool suppress_events, const Value& cookie)
{
	m_ShouldConnect = value;
	if (!suppress_events)
		NotifyShouldConnect(cookie);
}

void ObjectImpl<GelfWriter>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<GelfWriter *>(this), cookie);
}

void ObjectImpl<GelfWriter>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<GelfWriter *>(this), cookie);
}

void ObjectImpl<GelfWriter>::NotifySource(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSourceChanged(static_cast<GelfWriter *>(this), cookie);
}

void ObjectImpl<GelfWriter>::NotifyEnableSendPerfdata(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnEnableSendPerfdataChanged(static_cast<GelfWriter *>(this), cookie);
}

void ObjectImpl<GelfWriter>::NotifyConnected(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnConnectedChanged(static_cast<GelfWriter *>(this), cookie);
}

void ObjectImpl<GelfWriter>::NotifyShouldConnect(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnShouldConnectChanged(static_cast<GelfWriter *>(this), cookie);
}

String ObjectImpl<GelfWriter>::GetDefaultHost() const
{
	 return "127.0.0.1"; 
}

String ObjectImpl<GelfWriter>::GetDefaultPort() const
{
	 return "12201"; 
}

String ObjectImpl<GelfWriter>::GetDefaultSource() const
{
	 return "icinga2"; 
}

bool ObjectImpl<GelfWriter>::GetDefaultEnableSendPerfdata() const
{
	 return false; 
}

bool ObjectImpl<GelfWriter>::GetDefaultConnected() const
{
	return bool();
}

bool ObjectImpl<GelfWriter>::GetDefaultShouldConnect() const
{
	 return true; 
}


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnPortChanged;


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnSourceChanged;


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnEnableSendPerfdataChanged;


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnConnectedChanged;


boost::signals2::signal<void (const intrusive_ptr<GelfWriter>&, const Value&)> ObjectImpl<GelfWriter>::OnShouldConnectChanged;

void ObjectImpl<GelfWriter>::ValidateConnected(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateConnected(lvalue, utils);
}

void ObjectImpl<GelfWriter>::ValidateEnableSendPerfdata(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateEnableSendPerfdata(lvalue, utils);
}

void ObjectImpl<GelfWriter>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
}

void ObjectImpl<GelfWriter>::ValidatePort(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
}

void ObjectImpl<GelfWriter>::ValidateShouldConnect(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateShouldConnect(lvalue, utils);
}

void ObjectImpl<GelfWriter>::ValidateSource(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSource(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
